@@ -1,12 +1,14 @@
 import os
 import sys
 import shutil
-from lesscli import Application
+from lesscli import add_argument, add_subcommand, run
 import re
 import json
 import random
 from arms.utils.wordstyle import replace_dict, replace_all, WordStyle, WordSeed
 from arms.utils.common import dump_file_name
+from arms.designer import run_design
+from arms.codegen import run_codegen
 from pathlib import Path
 from rich.console import Console
 from rich.panel import Panel
@@ -72,19 +74,6 @@ def makedir(real_path):
     Path(real_path).mkdir(parents=True, exist_ok=True)
 
 
-def print_help():
-    text = """
-
-    arms init [git_url]         : 项目初始化（直接覆盖）
-    arms patch [git_url]        : 项目补丁（保留重名文件）
-    arms -h                     : show help information
-    arms -v                     : show version
-
-
-    """
-    print(text)
-
-
 def print_version():
     """
     显示版本
@@ -109,6 +98,8 @@ def run_process(tpl_name, is_patch=False):
     if not local_path.exists():
         alert_and_exit('请先执行arms config [git_url]')
     config_json = json.loads(local_path.open(encoding='utf-8').read())
+    if '__default__' in config_json:
+        config_json = config_json[config_json['__default__']]
     if tpl_name not in config_json:
         alert_and_exit('模版名称不存在！')
     ret = os.system('rm -rf .arms_tpl && git clone %s .arms_tpl' % config_json[tpl_name]['git_url'])
@@ -242,15 +233,15 @@ def run_process(tpl_name, is_patch=False):
         print('---- arms init succeed :) ----')
 
 
-def run_init(*args):
+@add_argument('tpl_name', help='模版名')
+def run_init(tpl_name):
     """
     项目初始化工具
-    arms init [name]
+
+    请输入命令完成项目初始化：arms init [tpl_name] （可先使用arms search [keyword]搜索模版名）
     """
-    if len(args) != 1:
-        alert_and_exit("请输入命令完成项目初始化：arms init [name] （可先使用arms search [keyword]搜索模版名）")
     try:
-        run_process(tpl_name=args[0], is_patch=False)
+        run_process(tpl_name=tpl_name, is_patch=False)
     except KeyboardInterrupt:
         print()
     finally:
@@ -258,15 +249,14 @@ def run_init(*args):
         os.system('rm -f .arms_tpl.tgz')
 
 
-def run_patch(*args):
+@add_argument('tpl_name', help='模版名')
+def run_patch(tpl_name):
     """
     项目补丁工具
-    arms patch [name]
+    arms patch [tpl_name]
     """
-    if len(args) != 1:
-        alert_and_exit("请输入命令完成项目补丁：arms patch [name]")
     try:
-        run_process(tpl_name=args[0], is_patch=True)
+        run_process(tpl_name=tpl_name, is_patch=True)
     except KeyboardInterrupt:
         print()
     finally:
@@ -274,6 +264,20 @@ def run_patch(*args):
         os.system('rm -f .arms_tpl.tgz')
 
 
+def switch_config():
+    local_path = Path.home() / '.arms_config.json'
+    local_git_src = json.loads(local_path.open(encoding='utf-8').read())
+    assert '__url__' not in local_git_src
+    options = [key for key in local_git_src.keys() if key != '__default__']
+    opt_choice = terminal_menu(options, lambda x: '(✔)' + x if x == local_git_src['__default__'] else '   ' + x)
+    if opt_choice is None:
+        exit(0)
+    local_git_src['__default__'] = opt_choice
+    with open(local_path, 'w') as f:
+        f.write(json.dumps(local_git_src, ensure_ascii=False, indent=4))
+    print("切换配置成功.")
+
+
 def pull_config(git_url):
     # [1/2]拉取模版源
     ret = os.system("git clone %s .arms_tpl_src" % git_url)
@@ -282,8 +286,17 @@ def pull_config(git_url):
     # [2/2]保存到本地目录
     local_path = Path.home() / '.arms_config.json'
     try:
+        try:
+            local_git_src = json.loads(local_path.open(encoding='utf-8').read())
+        except:
+            local_git_src = {}
+        if '__url__' in local_git_src:
+            local_git_src = {local_git_src['__url__']: local_git_src}  #兼容老版本
+        patch_git_src = json.loads(open('.arms_tpl_src/config.json').read())
+        local_git_src[patch_git_src['__url__']] = patch_git_src
+        local_git_src['__default__'] = patch_git_src['__url__']
         with open(local_path, 'w') as f:
-            f.write(open('.arms_tpl_src/config.json').read())
+            f.write(json.dumps(local_git_src, ensure_ascii=False, indent=4))
         print("更新配置成功.")
     except Exception:
         print(f"无法修改本地配置文件：{local_path}")
@@ -291,40 +304,47 @@ def pull_config(git_url):
         shutil.rmtree('.arms_tpl_src')
 
 
-def run_update(*args):
+def run_update():
     """
     arms更新配置
     arms update
     """
     local_path = Path.home() / '.arms_config.json'
     try:
-        git_url = json.loads(local_path.open(encoding='utf-8').read())['__url__']
+        config_json = json.loads(local_path.open(encoding='utf-8').read())
+        if '__default__' in config_json:
+            config_json = config_json[config_json['__default__']]
+        git_url = config_json['__url__']
         pull_config(git_url)
     except Exception:
         print(f"无法读取本地配置文件：{local_path}，请先执行arms config [git_url]")
 
 
-def run_config(*args):
+@add_argument('git_url', help='git源访问链接', required=False)
+def run_config(git_url):
     """
     arms配置工具
-    arms config [git_url]
+
+    arms config [git_url]  新增git源
+    arms config            切换git源
     """
-    if len(args) != 1:
-        alert_and_exit("请输入文档访问链接：arms config [git_url]")
-    git_url = args[0]
-    pull_config(git_url)
+    if git_url:
+        pull_config(git_url)
+    else:
+        switch_config()
 
 
-def run_search(*args):
+@add_argument('keyword', help='搜索关键词')
+def run_search(keyword):
     """
     项目搜索工具
     arms config [keyword]
     """
-    if len(args) != 1:
-        alert_and_exit("请输入搜索关键词：arms search [keyword]")
-    keyword = args[0].lower()
+    keyword = keyword.lower()
     local_path = Path.home() / '.arms_config.json'
     config_json = json.loads(local_path.open(encoding='utf-8').read())
+    if '__default__' in config_json:
+        config_json = config_json[config_json['__default__']]
     for name, setting in config_json.items():
         if name.startswith('__'):
             continue
@@ -332,14 +352,22 @@ def run_search(*args):
             console.print(f'[bold]{name}[/bold] - {setting["description"]}')
 
 
+@add_subcommand('version', print_version)
+@add_subcommand('init', run_init)
+@add_subcommand('patch', run_patch)
+@add_subcommand('config', run_config)
+@add_subcommand('search', run_search)
+@add_subcommand('update', run_update)
+@add_subcommand('design', run_design)  # 生成文本并复制
+@add_subcommand('codegen', run_codegen)  # 生成文件或修改文件
 def entrypoint():
+    """
+    armstrong
+    """
+    pass
+
+
+def main():
     if sys.version_info.major == 2 or sys.version_info.minor < 5:
         alert_and_exit('arms已不再支持python2，请安装python3.5+')
-    Application('armstrong')\
-        .add('version', print_version)\
-        .add('init', run_init) \
-        .add('patch', run_patch) \
-        .add('config', run_config) \
-        .add('search', run_search) \
-        .add('update', run_update) \
-        .run()
+    run(entrypoint)
